'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.bindSystem = bindSystem;
exports.loadSystem = loadSystem;
exports.unloadSystem = unloadSystem;

var _destroy = require('../lib/destroy');

var _state = require('../lib/state');

var _state2 = _interopRequireDefault(_state);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _pluginsSystem = [];
var _loadedSystem = false;

/**
 * @param {function}
 */
function bindSystem(createInstance) {
	return function (plugin) {
		var state = (0, _state2.default)();
		var pluginObj = {
			load: function load() {
				var reload = arguments.length <= 0 || arguments[0] === undefined ? false : arguments[0];

				if (this.loaded === reload) {
					this.loaded = true;
					(0, _destroy.destroyPlugin)(this);
					createInstance(this, plugin);
				}
			},

			loaded: false,
			instance: undefined,
			pluginArguments: [state],
			pluginArgumentsObj: { state: state }
		};

		_pluginsSystem.push(pluginObj);
		_loadedSystem && pluginObj.load(false);

		return {
			plugins: function plugins() {
				return [];
			},
			hotReload: function hotReload(_plugin) {
				plugin = _plugin;
				pluginObj.load(true);
			},
			willDispose: function willDispose() {
				pluginObj.loaded = false;
				(0, _destroy.destroyPlugin)(pluginObj);

				if (_pluginsSystem.indexOf(pluginObj) !== -1) {
					_pluginsSystem.splice(_pluginsSystem.indexOf(pluginObj), 1);
				}
			}
		};
	};
}

function loadSystem() {
	if (_loadedSystem === false) {
		_loadedSystem = true;
		_pluginsSystem.forEach(function (pluginObj) {
			return pluginObj.load(false);
		});
	}
}

function unloadSystem() {
	if (_loadedSystem) {
		_loadedSystem = false;
		_pluginsSystem.forEach(function (pluginObj) {
			return (0, _destroy.destroyPlugin)(pluginObj);
		});
	}
}